@@ -7,18 +7,12 @@ pub enum Token {
     FunctionLiteral(fn(WTokens) -> WTokens),
     Container(String),
     ContainerLiteral(String),
-    Parameter(FunctionParameter),
+    Parameter(Range),
     Atom(String),
     Special(String),
     Group(Vec<Token>),
 }
 
-#[derive(Debug, Clone, Eq, PartialEq)]
-pub enum FunctionParameter {
-    Exact(usize),
-    Remaining,
-}
-
 #[derive(Debug, Clone)]
 pub struct WCode {
     pub container: Option<String>,
@@ -26,6 +20,13 @@ pub struct WCode {
     pub default_case: WTokens,
 }
 
+#[derive(Debug, Clone, PartialEq)]
+pub enum Range {
+    Full(std::ops::RangeInclusive<usize>),
+    To(std::ops::RangeFrom<usize>),
+    From(std::ops::RangeTo<usize>)
+}
+
 #[derive(Debug, Clone)]
 pub enum WFuncVariant {
     Container(String),
@@ -33,5 +34,5 @@ pub enum WFuncVariant {
 }
 
 pub type WTokens = Vec<Token>;
-pub type WFunc = fn(WTokens) -> WTokens;
-pub type State = HashMap<String, Vec<(WTokens, WTokens)>>;
+pub(crate) type WFunc = fn(WTokens) -> WTokens;
+pub(crate) type State = HashMap<String, Vec<(WTokens, WTokens)>>;
