@@ -1,7 +1,4 @@
-use crate::evaluator::WEval;
-use crate::models::{FunctionParameter, State, Token, WFuncVariant, WTokens};
-
-type WFuncPair = (Option<(usize, WFuncVariant)>, Option<(usize, WFuncVariant)>);
+use crate::models::{Range, State, Token, WFuncVariant, WTokens};
 
 pub fn as_nums(arr: WTokens) -> Vec<f64> {
     arr.iter()
@@ -16,28 +13,18 @@ pub fn as_wcode(arr: Vec<f64>) -> WTokens {
     arr.iter().map(|&value| Token::Value(value)).collect()
 }
 
-pub fn outter_function(arr: &WTokens) -> WFuncPair {
+pub fn last_function(arr: &WTokens) -> Option<(usize, WFuncVariant)> {
     let reversed = arr.iter().rev();
 
-    let mut results: WFuncPair = (None, None);
-
-    for (i, token) in arr.iter().enumerate() {
-        match token {
-            Token::Function(value) => results.0 = Some((i, WFuncVariant::Function(*value))),
-            Token::Container(value) => {
-                results.0 = Some((i, WFuncVariant::Container(value.to_string())))
-            }
-            _ => continue,
-        }
-    }
+    let mut results: Option<(usize, WFuncVariant)> = None;
 
     for (i, token) in reversed.enumerate() {
         match token {
             Token::Function(value) => {
-                results.1 = Some((arr.len() - (i + 1), WFuncVariant::Function(*value)))
+                results = Some((arr.len() - (i + 1), WFuncVariant::Function(*value)))
             }
             Token::Container(value) => {
-                results.1 = Some((
+                results = Some((
                     arr.len() - (i + 1),
                     WFuncVariant::Container(value.to_string()),
                 ))
@@ -78,13 +65,13 @@ pub fn special_pairs(
     let mut counter = 0;
     let mut next_open = 0;
 
-    for (i, token) in arr[initial_pos.clone() + 1..].iter().enumerate() {
+    for (i, token) in arr[*initial_pos + 1..].iter().enumerate() {
         match token {
             Token::Special(value) => {
                 if value == &tokens.1 {
                     return Some(initial_pos + i + 1);
                 } else if value == &tokens.0 {
-                    next_open = initial_pos + i;
+                    next_open = initial_pos + i + 1;
                     counter += 1;
                     break;
                 }
@@ -131,89 +118,40 @@ pub fn first_special_instance(special: String, arr: &WTokens) -> Option<usize> {
 }
 
 pub trait WFunc {
-    fn apply(&self, function: &WTokens, arr: &WTokens, all_tokens_used: &WTokens) -> WTokens;
+    fn apply(&self, function: &WTokens, arr: &WTokens) -> WTokens;
 }
 
 impl WFunc for State {
-    fn apply(&self, function: &WTokens, arr: &WTokens, all_tokens_used: &WTokens) -> WTokens {
-        fn release_groups(arr: &WTokens) -> WTokens {
-            let mut released = arr.clone();
+    fn apply(&self, function: &WTokens, arr: &WTokens) -> WTokens {
+        let mut buffer = vec![];
+        let reversed: WTokens = arr.iter().cloned().rev().collect();
 
-            for (i, token) in arr.iter().enumerate() {
-                if let Token::Group(group) = token {
-                    released.splice(i..i + 1, group.clone());
-                    release_groups(&released);
+        for token in function {
+            match token {
+                Token::Parameter(range) => {
+                    let mut slice = match range {
+                        Range::From(from) => {
+                            reversed[*from].iter().cloned().rev().collect::<WTokens>()
+                        }
+                        Range::To(to) => reversed[to.clone()]
+                            .iter()
+                            .cloned()
+                            .rev()
+                            .collect::<WTokens>(),
+                        Range::Full(full) => reversed[full.clone()]
+                            .iter()
+                            .cloned()
+                            .rev()
+                            .collect::<WTokens>(),
+                    };
+
+                    buffer.append(&mut slice);
                 }
+                Token::Group(x) => buffer.push(Token::Group(self.apply(x, arr))),
+                _ => buffer.push(token.clone()),
             }
-
-            released
         }
 
-        fn parameters_used(arr: &WTokens) -> usize {
-            arr.iter().fold(0, |acc, x| match x {
-                Token::Parameter(FunctionParameter::Exact(x)) => {
-                    if acc < x + 1 {
-                        x + 1
-                    } else {
-                        acc
-                    }
-                }
-                _ => acc,
-            })
-        }
-
-        fn map_parameters(
-            mut buffer: WTokens,
-            function: &WTokens,
-            arr: &WTokens,
-            max_param: usize,
-        ) -> WTokens {
-            for token in function {
-                match token {
-                    Token::Parameter(FunctionParameter::Exact(x)) => {
-                        buffer.push(match arr.get(*x) {
-                            Some(y) => y.clone(),
-                            None => continue,
-                        })
-                    }
-                    Token::Parameter(FunctionParameter::Remaining) => {
-                        buffer.append(&mut arr[max_param..].to_vec())
-                    }
-                    Token::Group(x) => {
-                        buffer.push(Token::Group(map_parameters(vec![], x, arr, max_param)))
-                    }
-                    _ => buffer.push(token.clone()),
-                }
-            }
-
-            buffer
-        }
-
-        let released_function = release_groups(function);
-
-        let has_remaining_param = released_function.iter().any(|x| match x {
-            Token::Parameter(FunctionParameter::Remaining) => true,
-            _ => false,
-        });
-
-        let max_param: usize = parameters_used(&released_function);
-
-        let buffer = map_parameters(vec![], function, arr, max_param);
-
-        let mut result = self.eval(buffer);
-
-        if has_remaining_param {
-            result
-        } else {
-            let mut untouched = arr.clone();
-
-            let total_param = parameters_used(all_tokens_used);
-
-            for _ in 0..total_param {
-                untouched.pop();
-            }
-            untouched.append(&mut result);
-            untouched
-        }
+        buffer
     }
 }
